@@ -1,6 +1,3 @@
-// TODO(@gregorycooke) - Remove when only golang 1.19+ is supported
-//go:build go1.19
-
 /*
  *
  * Copyright 2021 gRPC authors.
@@ -29,7 +26,6 @@ import (
 	"crypto/x509"
 	"crypto/x509/pkix"
 	"encoding/asn1"
-	"encoding/hex"
 	"encoding/pem"
 	"fmt"
 	"math/big"
@@ -40,90 +36,9 @@ import (
 	"testing"
 	"time"
 
-	lru "github.com/hashicorp/golang-lru"
 	"google.golang.org/grpc/security/advancedtls/testdata"
 )
 
-func TestX509NameHash(t *testing.T) {
-	nameTests := []struct {
-		in  pkix.Name
-		out string
-	}{
-		{
-			in: pkix.Name{
-				Country:      []string{"US"},
-				Organization: []string{"Example"},
-			},
-			out: "9cdd41ff",
-		},
-		{
-			in: pkix.Name{
-				Country:      []string{"us"},
-				Organization: []string{"example"},
-			},
-			out: "9cdd41ff",
-		},
-		{
-			in: pkix.Name{
-				Country:      []string{"      us"},
-				Organization: []string{"example"},
-			},
-			out: "9cdd41ff",
-		},
-		{
-			in: pkix.Name{
-				Country:      []string{"US"},
-				Province:     []string{"California"},
-				Locality:     []string{"Mountain View"},
-				Organization: []string{"BoringSSL"},
-			},
-			out: "c24414d9",
-		},
-		{
-			in: pkix.Name{
-				Country:      []string{"US"},
-				Province:     []string{"California"},
-				Locality:     []string{"Mountain           View"},
-				Organization: []string{"BoringSSL"},
-			},
-			out: "c24414d9",
-		},
-		{
-			in: pkix.Name{
-				SerialNumber: "87f4514475ba0a2b",
-			},
-			out: "9dc713cd",
-		},
-		{
-			in: pkix.Name{
-				Country:            []string{"US"},
-				Province:           []string{"California"},
-				Locality:           []string{"Mountain View"},
-				Organization:       []string{"Google LLC"},
-				OrganizationalUnit: []string{"Production", "campus-sln"},
-				CommonName:         "Root CA (2021-02-02T07:30:36-08:00)",
-			},
-			out: "0b35a562",
-		},
-		{
-			in: pkix.Name{
-				ExtraNames: []pkix.AttributeTypeAndValue{
-					{Type: asn1.ObjectIdentifier{5, 5, 5, 5}, Value: "aaaa"},
-				},
-			},
-			out: "eea339da",
-		},
-	}
-	for _, tt := range nameTests {
-		t.Run(tt.in.String(), func(t *testing.T) {
-			h := x509NameHash(tt.in.ToRDNSequence())
-			if h != tt.out {
-				t.Errorf("x509NameHash(%v): Got %v wanted %v", tt.in, h, tt.out)
-			}
-		})
-	}
-}
-
 func TestUnsupportedCRLs(t *testing.T) {
 	crlBytesSomeReasons := []byte(`-----BEGIN X509 CRL-----
 MIIEeDCCA2ACAQEwDQYJKoZIhvcNAQELBQAwQjELMAkGA1UEBhMCVVMxHjAcBgNV
@@ -345,98 +260,8 @@ func loadCRL(t *testing.T, path string) *CRL {
 	return crl
 }
 
-func TestCachedCRL(t *testing.T) {
-	cache, err := lru.New(5)
-	if err != nil {
-		t.Fatalf("lru.New: err = %v", err)
-	}
-
-	tests := []struct {
-		desc string
-		val  any
-		ok   bool
-	}{
-		{
-			desc: "Valid",
-			val: &CRL{
-				certList: &x509.RevocationList{
-					NextUpdate: time.Now().Add(time.Hour),
-				}},
-			ok: true,
-		},
-		{
-			desc: "Expired",
-			val: &CRL{
-				certList: &x509.RevocationList{
-					NextUpdate: time.Now().Add(-time.Hour),
-				}},
-			ok: false,
-		},
-		{
-			desc: "Wrong Type",
-			val:  "string",
-			ok:   false,
-		},
-		{
-			desc: "Empty",
-			val:  nil,
-			ok:   false,
-		},
-	}
-	for _, tt := range tests {
-		t.Run(tt.desc, func(t *testing.T) {
-			if tt.val != nil {
-				cache.Add(hex.EncodeToString([]byte(tt.desc)), tt.val)
-			}
-			_, ok := cachedCrl([]byte(tt.desc), cache)
-			if tt.ok != ok {
-				t.Errorf("Cache ok error expected %v vs %v", tt.ok, ok)
-			}
-		})
-	}
-}
-
-func TestGetIssuerCRLCache(t *testing.T) {
-	cache, err := lru.New(5)
-	if err != nil {
-		t.Fatalf("lru.New: err = %v", err)
-	}
-
-	tests := []struct {
-		desc      string
-		rawIssuer []byte
-		certs     []*x509.Certificate
-	}{
-		{
-			desc:      "Valid",
-			rawIssuer: makeChain(t, testdata.Path("crl/unrevoked.pem"))[1].RawIssuer,
-			certs:     makeChain(t, testdata.Path("crl/unrevoked.pem")),
-		},
-		{
-			desc:      "Unverified",
-			rawIssuer: makeChain(t, testdata.Path("crl/unrevoked.pem"))[1].RawIssuer,
-		},
-		{
-			desc:      "Not Found",
-			rawIssuer: []byte("not_found"),
-		},
-	}
-
-	for _, tt := range tests {
-		t.Run(tt.desc, func(t *testing.T) {
-			cache.Purge()
-			_, err := fetchIssuerCRL(tt.rawIssuer, tt.certs, RevocationOptions{
-				RootDir: testdata.Path("."),
-				Cache:   cache,
-			})
-			if err == nil && cache.Len() == 0 {
-				t.Error("Verified CRL not added to cache")
-			}
-			if err != nil && cache.Len() != 0 {
-				t.Error("Unverified CRL added to cache")
-			}
-		})
-	}
+func checkRevocation(conn tls.ConnectionState, cfg RevocationOptions) error {
+	return checkChainRevocation(conn.VerifiedChains, cfg)
 }
 
 func TestVerifyCrl(t *testing.T) {
@@ -531,7 +356,6 @@ func TestRevokedCert(t *testing.T) {
 	revokedIntChain := makeChain(t, testdata.Path("crl/revokedInt.pem"))
 	revokedLeafChain := makeChain(t, testdata.Path("crl/revokedLeaf.pem"))
 	validChain := makeChain(t, testdata.Path("crl/unrevoked.pem"))
-	cache, err := lru.New(5)
 	rawCRLs := make([][]byte, 6)
 	for i := 1; i <= 6; i++ {
 		rawCRL, err := os.ReadFile(testdata.Path(fmt.Sprintf("crl/%d.crl", i)))
@@ -540,10 +364,12 @@ func TestRevokedCert(t *testing.T) {
 		}
 		rawCRLs = append(rawCRLs, rawCRL)
 	}
-	cRLProvider := NewStaticCRLProvider(rawCRLs)
+	staticCRLProvider := NewStaticCRLProvider(rawCRLs)
+	directoryCRLProvider, err := NewFileWatcherCRLProvider(FileWatcherOptions{CRLDirectory: testdata.Path("crl")})
 	if err != nil {
-		t.Fatalf("lru.New: err = %v", err)
+		t.Fatalf("NewFileWatcherCRLProvider: err = %v", err)
 	}
+	defer directoryCRLProvider.Close()
 
 	var revocationTests = []struct {
 		desc             string
@@ -599,11 +425,10 @@ func TestRevokedCert(t *testing.T) {
 	}
 
 	for _, tt := range revocationTests {
-		t.Run(fmt.Sprintf("%v with x509 crl hash dir", tt.desc), func(t *testing.T) {
+		t.Run(fmt.Sprintf("%v with x509 crl dir", tt.desc), func(t *testing.T) {
 			err := checkRevocation(tt.in, RevocationOptions{
-				RootDir:          testdata.Path("crl"),
+				CRLProvider:      directoryCRLProvider,
 				DenyUndetermined: tt.denyUndetermined,
-				Cache:            cache,
 			})
 			t.Logf("checkRevocation err = %v", err)
 			if tt.revoked && err == nil {
@@ -615,7 +440,7 @@ func TestRevokedCert(t *testing.T) {
 		t.Run(fmt.Sprintf("%v with static provider", tt.desc), func(t *testing.T) {
 			err := checkRevocation(tt.in, RevocationOptions{
 				DenyUndetermined: tt.denyUndetermined,
-				CRLProvider:      cRLProvider,
+				CRLProvider:      staticCRLProvider,
 			})
 			t.Logf("checkRevocation err = %v", err)
 			if tt.revoked && err == nil {
@@ -728,17 +553,22 @@ func TestVerifyConnection(t *testing.T) {
 				t.Fatalf("templ.CreateRevocationList failed err = %v", err)
 			}
 
-			err = os.WriteFile(path.Join(dir, fmt.Sprintf("%s.r0", x509NameHash(cert.Subject.ToRDNSequence()))), crl, 0777)
+			err = os.WriteFile(path.Join(dir, fmt.Sprintf("%s.r0", cert.Subject.ToRDNSequence())), crl, 0777)
 			if err != nil {
 				t.Fatalf("os.WriteFile failed err = %v", err)
 			}
 
 			cp := x509.NewCertPool()
 			cp.AddCert(cert)
+			provider, err := NewFileWatcherCRLProvider(FileWatcherOptions{CRLDirectory: dir})
+			if err != nil {
+				t.Errorf("NewFileWatcherCRLProvider: err = %v", err)
+			}
+			defer provider.Close()
 			cliCfg := tls.Config{
 				RootCAs: cp,
 				VerifyConnection: func(cs tls.ConnectionState) error {
-					return checkRevocation(cs, RevocationOptions{RootDir: dir})
+					return checkRevocation(cs, RevocationOptions{CRLProvider: provider})
 				},
 			}
 			conn, err := tls.Dial(lis.Addr().Network(), lis.Addr().String(), &cliCfg)
@@ -755,54 +585,3 @@ func TestVerifyConnection(t *testing.T) {
 		})
 	}
 }
-
-func TestIssuerNonPrintableString(t *testing.T) {
-	rawIssuer, err := hex.DecodeString("300c310a300806022a030c023a29")
-	if err != nil {
-		t.Fatalf("failed to decode issuer: %s", err)
-	}
-	_, err = fetchCRLOpenSSLHashDir(rawIssuer, RevocationOptions{RootDir: testdata.Path("crl")})
-	if err != nil {
-		t.Fatalf("fetchCRL failed: %s", err)
-	}
-}
-
-// TestCRLCacheExpirationReloading tests the basic expiration and reloading of a
-// cached CRL. The setup places an empty CRL in the cache, and a corresponding
-// CRL with a revocation in the CRL directory. We then validate the certificate
-// to verify that the certificate is not revoked.  Then, we modify the
-// NextUpdate time to be in the past so that when we next check for revocation,
-// the existing cache entry should be seen as expired, and the CRL in the
-// directory showing `revokedInt.pem` as revoked will be loaded, resulting in
-// the check returning `RevocationRevoked`.
-func TestCRLCacheExpirationReloading(t *testing.T) {
-	cache, err := lru.New(5)
-	if err != nil {
-		t.Fatalf("Creating cache failed")
-	}
-
-	var certs = makeChain(t, testdata.Path("crl/revokedInt.pem"))
-	// Certs[1] has the same issuer as the revoked cert
-	rawIssuer := certs[1].RawIssuer
-
-	// `3.crl`` revokes `revokedInt.pem`
-	crl := loadCRL(t, testdata.Path("crl/3.crl"))
-	// Modify the crl so that the cert is NOT revoked and add it to the cache
-	crl.certList.RevokedCertificates = nil
-	crl.certList.NextUpdate = time.Now().Add(time.Hour)
-	cache.Add(hex.EncodeToString(rawIssuer), crl)
-	var cfg = RevocationOptions{RootDir: testdata.Path("crl"), Cache: cache}
-	revocationStatus := checkChain(certs, cfg)
-	if revocationStatus != RevocationUnrevoked {
-		t.Fatalf("Certificate check should be RevocationUnrevoked, was %v", revocationStatus)
-	}
-
-	// Modify the entry in the cache so that the cache will be refreshed
-	crl.certList.NextUpdate = time.Now()
-	cache.Add(hex.EncodeToString(rawIssuer), crl)
-
-	revocationStatus = checkChain(certs, cfg)
-	if revocationStatus != RevocationRevoked {
-		t.Fatalf("A certificate should have been `RevocationRevoked` but was %v", revocationStatus)
-	}
-}
