@@ -28,15 +28,25 @@ import (
 	"testing"
 	"time"
 
-	"github.com/golang/protobuf/proto"
+	"github.com/google/go-cmp/cmp"
 	"google.golang.org/grpc"
+	"google.golang.org/grpc/credentials/insecure"
+	"google.golang.org/grpc/internal"
 	"google.golang.org/grpc/internal/grpctest"
+	"google.golang.org/grpc/internal/stubserver"
+	"google.golang.org/grpc/internal/testutils"
 	"google.golang.org/grpc/metadata"
 	"google.golang.org/grpc/stats"
-	testpb "google.golang.org/grpc/stats/grpc_testing"
 	"google.golang.org/grpc/status"
+	"google.golang.org/protobuf/proto"
+	"google.golang.org/protobuf/testing/protocmp"
+
+	testgrpc "google.golang.org/grpc/interop/grpc_testing"
+	testpb "google.golang.org/grpc/interop/grpc_testing"
 )
 
+const defaultTestTimeout = 10 * time.Second
+
 type s struct {
 	grpctest.Tester
 }
@@ -73,8 +83,19 @@ var (
 	errorID int32 = 32202
 )
 
+func idToPayload(id int32) *testpb.Payload {
+	return &testpb.Payload{Body: []byte{byte(id), byte(id >> 8), byte(id >> 16), byte(id >> 24)}}
+}
+
+func payloadToID(p *testpb.Payload) int32 {
+	if p == nil || len(p.Body) != 4 {
+		panic("invalid payload")
+	}
+	return int32(p.Body[0]) + int32(p.Body[1])<<8 + int32(p.Body[2])<<16 + int32(p.Body[3])<<24
+}
+
 type testServer struct {
-	testpb.UnimplementedTestServiceServer
+	testgrpc.UnimplementedTestServiceServer
 }
 
 func (s *testServer) UnaryCall(ctx context.Context, in *testpb.SimpleRequest) (*testpb.SimpleResponse, error) {
@@ -85,14 +106,14 @@ func (s *testServer) UnaryCall(ctx context.Context, in *testpb.SimpleRequest) (*
 		return nil, status.Errorf(status.Code(err), "grpc.SetTrailer(_, %v) = %v, want <nil>", testTrailerMetadata, err)
 	}
 
-	if in.Id == errorID {
-		return nil, fmt.Errorf("got error id: %v", in.Id)
+	if id := payloadToID(in.Payload); id == errorID {
+		return nil, fmt.Errorf("got error id: %v", id)
 	}
 
-	return &testpb.SimpleResponse{Id: in.Id}, nil
+	return &testpb.SimpleResponse{Payload: in.Payload}, nil
 }
 
-func (s *testServer) FullDuplexCall(stream testpb.TestService_FullDuplexCallServer) error {
+func (s *testServer) FullDuplexCall(stream testgrpc.TestService_FullDuplexCallServer) error {
 	if err := stream.SendHeader(testHeaderMetadata); err != nil {
 		return status.Errorf(status.Code(err), "%v.SendHeader(%v) = %v, want %v", stream, testHeaderMetadata, err, nil)
 	}
@@ -107,17 +128,17 @@ func (s *testServer) FullDuplexCall(stream testpb.TestService_FullDuplexCallServ
 			return err
 		}
 
-		if in.Id == errorID {
-			return fmt.Errorf("got error id: %v", in.Id)
+		if id := payloadToID(in.Payload); id == errorID {
+			return fmt.Errorf("got error id: %v", id)
 		}
 
-		if err := stream.Send(&testpb.SimpleResponse{Id: in.Id}); err != nil {
+		if err := stream.Send(&testpb.StreamingOutputCallResponse{Payload: in.Payload}); err != nil {
 			return err
 		}
 	}
 }
 
-func (s *testServer) ClientStreamCall(stream testpb.TestService_ClientStreamCallServer) error {
+func (s *testServer) StreamingInputCall(stream testgrpc.TestService_StreamingInputCallServer) error {
 	if err := stream.SendHeader(testHeaderMetadata); err != nil {
 		return status.Errorf(status.Code(err), "%v.SendHeader(%v) = %v, want %v", stream, testHeaderMetadata, err, nil)
 	}
@@ -126,30 +147,30 @@ func (s *testServer) ClientStreamCall(stream testpb.TestService_ClientStreamCall
 		in, err := stream.Recv()
 		if err == io.EOF {
 			// read done.
-			return stream.SendAndClose(&testpb.SimpleResponse{Id: int32(0)})
+			return stream.SendAndClose(&testpb.StreamingInputCallResponse{AggregatedPayloadSize: 0})
 		}
 		if err != nil {
 			return err
 		}
 
-		if in.Id == errorID {
-			return fmt.Errorf("got error id: %v", in.Id)
+		if id := payloadToID(in.Payload); id == errorID {
+			return fmt.Errorf("got error id: %v", id)
 		}
 	}
 }
 
-func (s *testServer) ServerStreamCall(in *testpb.SimpleRequest, stream testpb.TestService_ServerStreamCallServer) error {
+func (s *testServer) StreamingOutputCall(in *testpb.StreamingOutputCallRequest, stream testgrpc.TestService_StreamingOutputCallServer) error {
 	if err := stream.SendHeader(testHeaderMetadata); err != nil {
 		return status.Errorf(status.Code(err), "%v.SendHeader(%v) = %v, want %v", stream, testHeaderMetadata, err, nil)
 	}
 	stream.SetTrailer(testTrailerMetadata)
 
-	if in.Id == errorID {
-		return fmt.Errorf("got error id: %v", in.Id)
+	if id := payloadToID(in.Payload); id == errorID {
+		return fmt.Errorf("got error id: %v", id)
 	}
 
 	for i := 0; i < 5; i++ {
-		if err := stream.Send(&testpb.SimpleResponse{Id: in.Id}); err != nil {
+		if err := stream.Send(&testpb.StreamingOutputCallResponse{Payload: in.Payload}); err != nil {
 			return err
 		}
 	}
@@ -160,12 +181,12 @@ func (s *testServer) ServerStreamCall(in *testpb.SimpleRequest, stream testpb.Te
 // func, modified as needed, and then started with its startServer method.
 // It should be cleaned up with the tearDown method.
 type test struct {
-	t                  *testing.T
-	compress           string
-	clientStatsHandler stats.Handler
-	serverStatsHandler stats.Handler
+	t                   *testing.T
+	compress            string
+	clientStatsHandlers []stats.Handler
+	serverStatsHandlers []stats.Handler
 
-	testServer testpb.TestServiceServer // nil means none
+	testServer testgrpc.TestServiceServer // nil means none
 	// srv and srvAddr are set once startServer is called.
 	srv     *grpc.Server
 	srvAddr string
@@ -188,19 +209,19 @@ type testConfig struct {
 // newTest returns a new test using the provided testing.T and
 // environment.  It is returned with default values. Tests should
 // modify it before calling its startServer and clientConn methods.
-func newTest(t *testing.T, tc *testConfig, ch stats.Handler, sh stats.Handler) *test {
+func newTest(t *testing.T, tc *testConfig, chs []stats.Handler, shs []stats.Handler) *test {
 	te := &test{
-		t:                  t,
-		compress:           tc.compress,
-		clientStatsHandler: ch,
-		serverStatsHandler: sh,
+		t:                   t,
+		compress:            tc.compress,
+		clientStatsHandlers: chs,
+		serverStatsHandlers: shs,
 	}
 	return te
 }
 
 // startServer starts a gRPC server listening. Callers should defer a
 // call to te.tearDown to clean up.
-func (te *test) startServer(ts testpb.TestServiceServer) {
+func (te *test) startServer(ts testgrpc.TestServiceServer) {
 	te.testServer = ts
 	lis, err := net.Listen("tcp", "localhost:0")
 	if err != nil {
@@ -213,13 +234,13 @@ func (te *test) startServer(ts testpb.TestServiceServer) {
 			grpc.RPCDecompressor(grpc.NewGZIPDecompressor()),
 		)
 	}
-	if te.serverStatsHandler != nil {
-		opts = append(opts, grpc.StatsHandler(te.serverStatsHandler))
+	for _, sh := range te.serverStatsHandlers {
+		opts = append(opts, grpc.StatsHandler(sh))
 	}
 	s := grpc.NewServer(opts...)
 	te.srv = s
 	if te.testServer != nil {
-		testpb.RegisterTestServiceServer(s, te.testServer)
+		testgrpc.RegisterTestServiceServer(s, te.testServer)
 	}
 
 	go s.Serve(lis)
@@ -231,7 +252,7 @@ func (te *test) clientConn() *grpc.ClientConn {
 		return te.cc
 	}
 	opts := []grpc.DialOption{
-		grpc.WithInsecure(),
+		grpc.WithTransportCredentials(insecure.NewCredentials()),
 		grpc.WithBlock(),
 		grpc.WithUserAgent("test/0.0.1"),
 	}
@@ -241,8 +262,8 @@ func (te *test) clientConn() *grpc.ClientConn {
 			grpc.WithDecompressor(grpc.NewGZIPDecompressor()),
 		)
 	}
-	if te.clientStatsHandler != nil {
-		opts = append(opts, grpc.WithStatsHandler(te.clientStatsHandler))
+	for _, sh := range te.clientStatsHandlers {
+		opts = append(opts, grpc.WithStatsHandler(sh))
 	}
 
 	var err error
@@ -275,25 +296,29 @@ func (te *test) doUnaryCall(c *rpcConfig) (*testpb.SimpleRequest, *testpb.Simple
 		req  *testpb.SimpleRequest
 		err  error
 	)
-	tc := testpb.NewTestServiceClient(te.clientConn())
+	tc := testgrpc.NewTestServiceClient(te.clientConn())
 	if c.success {
-		req = &testpb.SimpleRequest{Id: errorID + 1}
+		req = &testpb.SimpleRequest{Payload: idToPayload(errorID + 1)}
 	} else {
-		req = &testpb.SimpleRequest{Id: errorID}
+		req = &testpb.SimpleRequest{Payload: idToPayload(errorID)}
 	}
-	ctx := metadata.NewOutgoingContext(context.Background(), testMetadata)
-	resp, err = tc.UnaryCall(ctx, req, grpc.WaitForReady(!c.failfast))
+
+	tCtx, cancel := context.WithTimeout(context.Background(), defaultTestTimeout)
+	defer cancel()
+	resp, err = tc.UnaryCall(metadata.NewOutgoingContext(tCtx, testMetadata), req, grpc.WaitForReady(!c.failfast))
 	return req, resp, err
 }
 
-func (te *test) doFullDuplexCallRoundtrip(c *rpcConfig) ([]*testpb.SimpleRequest, []*testpb.SimpleResponse, error) {
+func (te *test) doFullDuplexCallRoundtrip(c *rpcConfig) ([]proto.Message, []proto.Message, error) {
 	var (
-		reqs  []*testpb.SimpleRequest
-		resps []*testpb.SimpleResponse
+		reqs  []proto.Message
+		resps []proto.Message
 		err   error
 	)
-	tc := testpb.NewTestServiceClient(te.clientConn())
-	stream, err := tc.FullDuplexCall(metadata.NewOutgoingContext(context.Background(), testMetadata), grpc.WaitForReady(!c.failfast))
+	tc := testgrpc.NewTestServiceClient(te.clientConn())
+	tCtx, cancel := context.WithTimeout(context.Background(), defaultTestTimeout)
+	defer cancel()
+	stream, err := tc.FullDuplexCall(metadata.NewOutgoingContext(tCtx, testMetadata), grpc.WaitForReady(!c.failfast))
 	if err != nil {
 		return reqs, resps, err
 	}
@@ -302,14 +327,14 @@ func (te *test) doFullDuplexCallRoundtrip(c *rpcConfig) ([]*testpb.SimpleRequest
 		startID = errorID
 	}
 	for i := 0; i < c.count; i++ {
-		req := &testpb.SimpleRequest{
-			Id: int32(i) + startID,
+		req := &testpb.StreamingOutputCallRequest{
+			Payload: idToPayload(int32(i) + startID),
 		}
 		reqs = append(reqs, req)
 		if err = stream.Send(req); err != nil {
 			return reqs, resps, err
 		}
-		var resp *testpb.SimpleResponse
+		var resp *testpb.StreamingOutputCallResponse
 		if resp, err = stream.Recv(); err != nil {
 			return reqs, resps, err
 		}
@@ -325,14 +350,16 @@ func (te *test) doFullDuplexCallRoundtrip(c *rpcConfig) ([]*testpb.SimpleRequest
 	return reqs, resps, nil
 }
 
-func (te *test) doClientStreamCall(c *rpcConfig) ([]*testpb.SimpleRequest, *testpb.SimpleResponse, error) {
+func (te *test) doClientStreamCall(c *rpcConfig) ([]proto.Message, *testpb.StreamingInputCallResponse, error) {
 	var (
-		reqs []*testpb.SimpleRequest
-		resp *testpb.SimpleResponse
+		reqs []proto.Message
+		resp *testpb.StreamingInputCallResponse
 		err  error
 	)
-	tc := testpb.NewTestServiceClient(te.clientConn())
-	stream, err := tc.ClientStreamCall(metadata.NewOutgoingContext(context.Background(), testMetadata), grpc.WaitForReady(!c.failfast))
+	tc := testgrpc.NewTestServiceClient(te.clientConn())
+	tCtx, cancel := context.WithTimeout(context.Background(), defaultTestTimeout)
+	defer cancel()
+	stream, err := tc.StreamingInputCall(metadata.NewOutgoingContext(tCtx, testMetadata), grpc.WaitForReady(!c.failfast))
 	if err != nil {
 		return reqs, resp, err
 	}
@@ -341,8 +368,8 @@ func (te *test) doClientStreamCall(c *rpcConfig) ([]*testpb.SimpleRequest, *test
 		startID = errorID
 	}
 	for i := 0; i < c.count; i++ {
-		req := &testpb.SimpleRequest{
-			Id: int32(i) + startID,
+		req := &testpb.StreamingInputCallRequest{
+			Payload: idToPayload(int32(i) + startID),
 		}
 		reqs = append(reqs, req)
 		if err = stream.Send(req); err != nil {
@@ -353,26 +380,28 @@ func (te *test) doClientStreamCall(c *rpcConfig) ([]*testpb.SimpleRequest, *test
 	return reqs, resp, err
 }
 
-func (te *test) doServerStreamCall(c *rpcConfig) (*testpb.SimpleRequest, []*testpb.SimpleResponse, error) {
+func (te *test) doServerStreamCall(c *rpcConfig) (*testpb.StreamingOutputCallRequest, []proto.Message, error) {
 	var (
-		req   *testpb.SimpleRequest
-		resps []*testpb.SimpleResponse
+		req   *testpb.StreamingOutputCallRequest
+		resps []proto.Message
 		err   error
 	)
 
-	tc := testpb.NewTestServiceClient(te.clientConn())
+	tc := testgrpc.NewTestServiceClient(te.clientConn())
 
 	var startID int32
 	if !c.success {
 		startID = errorID
 	}
-	req = &testpb.SimpleRequest{Id: startID}
-	stream, err := tc.ServerStreamCall(metadata.NewOutgoingContext(context.Background(), testMetadata), req, grpc.WaitForReady(!c.failfast))
+	req = &testpb.StreamingOutputCallRequest{Payload: idToPayload(startID)}
+	tCtx, cancel := context.WithTimeout(context.Background(), defaultTestTimeout)
+	defer cancel()
+	stream, err := tc.StreamingOutputCall(metadata.NewOutgoingContext(tCtx, testMetadata), req, grpc.WaitForReady(!c.failfast))
 	if err != nil {
 		return req, resps, err
 	}
 	for {
-		var resp *testpb.SimpleResponse
+		var resp *testpb.StreamingOutputCallResponse
 		resp, err := stream.Recv()
 		if err == io.EOF {
 			return req, resps, nil
@@ -384,21 +413,23 @@ func (te *test) doServerStreamCall(c *rpcConfig) (*testpb.SimpleRequest, []*test
 }
 
 type expectedData struct {
-	method      string
-	serverAddr  string
-	compression string
-	reqIdx      int
-	requests    []*testpb.SimpleRequest
-	respIdx     int
-	responses   []*testpb.SimpleResponse
-	err         error
-	failfast    bool
+	method         string
+	isClientStream bool
+	isServerStream bool
+	serverAddr     string
+	compression    string
+	reqIdx         int
+	requests       []proto.Message
+	respIdx        int
+	responses      []proto.Message
+	err            error
+	failfast       bool
 }
 
 type gotData struct {
 	ctx    context.Context
 	client bool
-	s      interface{} // This could be RPCStats or ConnStats.
+	s      any // This could be RPCStats or ConnStats.
 }
 
 const (
@@ -433,6 +464,12 @@ func checkBegin(t *testing.T, d *gotData, e *expectedData) {
 			t.Fatalf("st.FailFast = %v, want %v", st.FailFast, e.failfast)
 		}
 	}
+	if st.IsClientStream != e.isClientStream {
+		t.Fatalf("st.IsClientStream = %v, want %v", st.IsClientStream, e.isClientStream)
+	}
+	if st.IsServerStream != e.isServerStream {
+		t.Fatalf("st.IsServerStream = %v, want %v", st.IsServerStream, e.isServerStream)
+	}
 }
 
 func checkInHeader(t *testing.T, d *gotData, e *expectedData) {
@@ -503,42 +540,31 @@ func checkInPayload(t *testing.T, d *gotData, e *expectedData) {
 	if d.ctx == nil {
 		t.Fatalf("d.ctx = nil, want <non-nil>")
 	}
+
+	var idx *int
+	var payloads []proto.Message
 	if d.client {
-		b, err := proto.Marshal(e.responses[e.respIdx])
-		if err != nil {
-			t.Fatalf("failed to marshal message: %v", err)
-		}
-		if reflect.TypeOf(st.Payload) != reflect.TypeOf(e.responses[e.respIdx]) {
-			t.Fatalf("st.Payload = %T, want %T", st.Payload, e.responses[e.respIdx])
-		}
-		e.respIdx++
-		if string(st.Data) != string(b) {
-			t.Fatalf("st.Data = %v, want %v", st.Data, b)
-		}
-		if st.Length != len(b) {
-			t.Fatalf("st.Lenght = %v, want %v", st.Length, len(b))
-		}
+		idx = &e.respIdx
+		payloads = e.responses
 	} else {
-		b, err := proto.Marshal(e.requests[e.reqIdx])
-		if err != nil {
-			t.Fatalf("failed to marshal message: %v", err)
-		}
-		if reflect.TypeOf(st.Payload) != reflect.TypeOf(e.requests[e.reqIdx]) {
-			t.Fatalf("st.Payload = %T, want %T", st.Payload, e.requests[e.reqIdx])
-		}
-		e.reqIdx++
-		if string(st.Data) != string(b) {
-			t.Fatalf("st.Data = %v, want %v", st.Data, b)
-		}
-		if st.Length != len(b) {
-			t.Fatalf("st.Lenght = %v, want %v", st.Length, len(b))
-		}
+		idx = &e.reqIdx
+		payloads = e.requests
 	}
+
+	wantPayload := payloads[*idx]
+	if diff := cmp.Diff(wantPayload, st.Payload.(proto.Message), protocmp.Transform()); diff != "" {
+		t.Fatalf("unexpected difference in st.Payload (-want +got):\n%s", diff)
+	}
+	*idx++
+	if st.Length != proto.Size(wantPayload) {
+		t.Fatalf("st.Length = %v, want %v", st.Length, proto.Size(wantPayload))
+	}
+
 	// Below are sanity checks that WireLength and RecvTime are populated.
 	// TODO: check values of WireLength and RecvTime.
-	if len(st.Data) > 0 && st.WireLength == 0 {
+	if st.Length > 0 && st.CompressedLength == 0 {
 		t.Fatalf("st.WireLength = %v with non-empty data, want <non-zero>",
-			st.WireLength)
+			st.CompressedLength)
 	}
 	if st.RecvTime.IsZero() {
 		t.Fatalf("st.ReceivedTime = %v, want <non-zero>", st.RecvTime)
@@ -622,40 +648,29 @@ func checkOutPayload(t *testing.T, d *gotData, e *expectedData) {
 	if d.ctx == nil {
 		t.Fatalf("d.ctx = nil, want <non-nil>")
 	}
+
+	var idx *int
+	var payloads []proto.Message
 	if d.client {
-		b, err := proto.Marshal(e.requests[e.reqIdx])
-		if err != nil {
-			t.Fatalf("failed to marshal message: %v", err)
-		}
-		if reflect.TypeOf(st.Payload) != reflect.TypeOf(e.requests[e.reqIdx]) {
-			t.Fatalf("st.Payload = %T, want %T", st.Payload, e.requests[e.reqIdx])
-		}
-		e.reqIdx++
-		if string(st.Data) != string(b) {
-			t.Fatalf("st.Data = %v, want %v", st.Data, b)
-		}
-		if st.Length != len(b) {
-			t.Fatalf("st.Lenght = %v, want %v", st.Length, len(b))
-		}
+		idx = &e.reqIdx
+		payloads = e.requests
 	} else {
-		b, err := proto.Marshal(e.responses[e.respIdx])
-		if err != nil {
-			t.Fatalf("failed to marshal message: %v", err)
-		}
-		if reflect.TypeOf(st.Payload) != reflect.TypeOf(e.responses[e.respIdx]) {
-			t.Fatalf("st.Payload = %T, want %T", st.Payload, e.responses[e.respIdx])
-		}
-		e.respIdx++
-		if string(st.Data) != string(b) {
-			t.Fatalf("st.Data = %v, want %v", st.Data, b)
-		}
-		if st.Length != len(b) {
-			t.Fatalf("st.Lenght = %v, want %v", st.Length, len(b))
-		}
+		idx = &e.respIdx
+		payloads = e.responses
 	}
-	// Below are sanity checks that WireLength and SentTime are populated.
+
+	expectedPayload := payloads[*idx]
+	if !proto.Equal(st.Payload.(proto.Message), expectedPayload) {
+		t.Fatalf("st.Payload = %v, want %v", st.Payload, expectedPayload)
+	}
+	*idx++
+	if st.Length != proto.Size(expectedPayload) {
+		t.Fatalf("st.Length = %v, want %v", st.Length, proto.Size(expectedPayload))
+	}
+
+	// Below are sanity checks that Length, CompressedLength and SentTime are populated.
 	// TODO: check values of WireLength and SentTime.
-	if len(st.Data) > 0 && st.WireLength == 0 {
+	if st.Length > 0 && st.WireLength == 0 {
 		t.Fatalf("st.WireLength = %v with non-empty data, want <non-zero>",
 			st.WireLength)
 	}
@@ -722,7 +737,7 @@ func checkEnd(t *testing.T, d *gotData, e *expectedData) {
 	}
 }
 
-func checkConnBegin(t *testing.T, d *gotData, e *expectedData) {
+func checkConnBegin(t *testing.T, d *gotData) {
 	var (
 		ok bool
 		st *stats.ConnBegin
@@ -736,7 +751,7 @@ func checkConnBegin(t *testing.T, d *gotData, e *expectedData) {
 	st.IsClient() // TODO remove this.
 }
 
-func checkConnEnd(t *testing.T, d *gotData, e *expectedData) {
+func checkConnEnd(t *testing.T, d *gotData) {
 	var (
 		ok bool
 		st *stats.ConnEnd
@@ -784,9 +799,9 @@ func checkConnStats(t *testing.T, got []*gotData) {
 		t.Fatalf("got %v stats, want even positive number", len(got))
 	}
 	// The first conn stats must be a ConnBegin.
-	checkConnBegin(t, got[0], nil)
+	checkConnBegin(t, got[0])
 	// The last conn stats must be a ConnEnd.
-	checkConnEnd(t, got[len(got)-1], nil)
+	checkConnEnd(t, got[len(got)-1])
 }
 
 func checkServerStats(t *testing.T, got []*gotData, expect *expectedData, checkFuncs []func(t *testing.T, d *gotData, e *expectedData)) {
@@ -797,16 +812,6 @@ func checkServerStats(t *testing.T, got []*gotData, expect *expectedData, checkF
 		t.Fatalf("got %v stats, want %v stats", len(got), len(checkFuncs))
 	}
 
-	var rpcctx context.Context
-	for i := 0; i < len(got); i++ {
-		if _, ok := got[i].s.(stats.RPCStats); ok {
-			if rpcctx != nil && got[i].ctx != rpcctx {
-				t.Fatalf("got different contexts with stats %T", got[i].s)
-			}
-			rpcctx = got[i].ctx
-		}
-	}
-
 	for i, f := range checkFuncs {
 		f(t, got[i], expect)
 	}
@@ -814,18 +819,21 @@ func checkServerStats(t *testing.T, got []*gotData, expect *expectedData, checkF
 
 func testServerStats(t *testing.T, tc *testConfig, cc *rpcConfig, checkFuncs []func(t *testing.T, d *gotData, e *expectedData)) {
 	h := &statshandler{}
-	te := newTest(t, tc, nil, h)
+	te := newTest(t, tc, nil, []stats.Handler{h})
 	te.startServer(&testServer{})
 	defer te.tearDown()
 
 	var (
-		reqs   []*testpb.SimpleRequest
-		resps  []*testpb.SimpleResponse
+		reqs   []proto.Message
+		resps  []proto.Message
 		err    error
 		method string
 
-		req  *testpb.SimpleRequest
-		resp *testpb.SimpleResponse
+		isClientStream bool
+		isServerStream bool
+
+		req  proto.Message
+		resp proto.Message
 		e    error
 	)
 
@@ -833,22 +841,26 @@ func testServerStats(t *testing.T, tc *testConfig, cc *rpcConfig, checkFuncs []f
 	case unaryRPC:
 		method = "/grpc.testing.TestService/UnaryCall"
 		req, resp, e = te.doUnaryCall(cc)
-		reqs = []*testpb.SimpleRequest{req}
-		resps = []*testpb.SimpleResponse{resp}
+		reqs = []proto.Message{req}
+		resps = []proto.Message{resp}
 		err = e
 	case clientStreamRPC:
-		method = "/grpc.testing.TestService/ClientStreamCall"
+		method = "/grpc.testing.TestService/StreamingInputCall"
 		reqs, resp, e = te.doClientStreamCall(cc)
-		resps = []*testpb.SimpleResponse{resp}
+		resps = []proto.Message{resp}
 		err = e
+		isClientStream = true
 	case serverStreamRPC:
-		method = "/grpc.testing.TestService/ServerStreamCall"
+		method = "/grpc.testing.TestService/StreamingOutputCall"
 		req, resps, e = te.doServerStreamCall(cc)
-		reqs = []*testpb.SimpleRequest{req}
+		reqs = []proto.Message{req}
 		err = e
+		isServerStream = true
 	case fullDuplexStreamRPC:
 		method = "/grpc.testing.TestService/FullDuplexCall"
 		reqs, resps, err = te.doFullDuplexCallRoundtrip(cc)
+		isClientStream = true
+		isServerStream = true
 	}
 	if cc.success != (err == nil) {
 		t.Fatalf("cc.success: %v, got error: %v", cc.success, err)
@@ -877,12 +889,14 @@ func testServerStats(t *testing.T, tc *testConfig, cc *rpcConfig, checkFuncs []f
 	}
 
 	expect := &expectedData{
-		serverAddr:  te.srvAddr,
-		compression: tc.compress,
-		method:      method,
-		requests:    reqs,
-		responses:   resps,
-		err:         err,
+		serverAddr:     te.srvAddr,
+		compression:    tc.compress,
+		method:         method,
+		requests:       reqs,
+		responses:      resps,
+		err:            err,
+		isClientStream: isClientStream,
+		isServerStream: isServerStream,
 	}
 
 	h.mu.Lock()
@@ -1105,40 +1119,47 @@ func checkClientStats(t *testing.T, got []*gotData, expect *expectedData, checkF
 
 func testClientStats(t *testing.T, tc *testConfig, cc *rpcConfig, checkFuncs map[int]*checkFuncWithCount) {
 	h := &statshandler{}
-	te := newTest(t, tc, h, nil)
+	te := newTest(t, tc, []stats.Handler{h}, nil)
 	te.startServer(&testServer{})
 	defer te.tearDown()
 
 	var (
-		reqs   []*testpb.SimpleRequest
-		resps  []*testpb.SimpleResponse
+		reqs   []proto.Message
+		resps  []proto.Message
 		method string
 		err    error
 
-		req  *testpb.SimpleRequest
-		resp *testpb.SimpleResponse
+		isClientStream bool
+		isServerStream bool
+
+		req  proto.Message
+		resp proto.Message
 		e    error
 	)
 	switch cc.callType {
 	case unaryRPC:
 		method = "/grpc.testing.TestService/UnaryCall"
 		req, resp, e = te.doUnaryCall(cc)
-		reqs = []*testpb.SimpleRequest{req}
-		resps = []*testpb.SimpleResponse{resp}
+		reqs = []proto.Message{req}
+		resps = []proto.Message{resp}
 		err = e
 	case clientStreamRPC:
-		method = "/grpc.testing.TestService/ClientStreamCall"
+		method = "/grpc.testing.TestService/StreamingInputCall"
 		reqs, resp, e = te.doClientStreamCall(cc)
-		resps = []*testpb.SimpleResponse{resp}
+		resps = []proto.Message{resp}
 		err = e
+		isClientStream = true
 	case serverStreamRPC:
-		method = "/grpc.testing.TestService/ServerStreamCall"
+		method = "/grpc.testing.TestService/StreamingOutputCall"
 		req, resps, e = te.doServerStreamCall(cc)
-		reqs = []*testpb.SimpleRequest{req}
+		reqs = []proto.Message{req}
 		err = e
+		isServerStream = true
 	case fullDuplexStreamRPC:
 		method = "/grpc.testing.TestService/FullDuplexCall"
 		reqs, resps, err = te.doFullDuplexCallRoundtrip(cc)
+		isClientStream = true
+		isServerStream = true
 	}
 	if cc.success != (err == nil) {
 		t.Fatalf("cc.success: %v, got error: %v", cc.success, err)
@@ -1171,13 +1192,15 @@ func testClientStats(t *testing.T, tc *testConfig, cc *rpcConfig, checkFuncs map
 	}
 
 	expect := &expectedData{
-		serverAddr:  te.srvAddr,
-		compression: tc.compress,
-		method:      method,
-		requests:    reqs,
-		responses:   resps,
-		failfast:    cc.failfast,
-		err:         err,
+		serverAddr:     te.srvAddr,
+		compression:    tc.compress,
+		method:         method,
+		requests:       reqs,
+		responses:      resps,
+		failfast:       cc.failfast,
+		err:            err,
+		isClientStream: isClientStream,
+		isServerStream: isServerStream,
 	}
 
 	h.mu.Lock()
@@ -1286,7 +1309,9 @@ func (s) TestClientStatsFullDuplexRPCError(t *testing.T) {
 
 func (s) TestTags(t *testing.T) {
 	b := []byte{5, 2, 4, 3, 1}
-	ctx := stats.SetTags(context.Background(), b)
+	tCtx, cancel := context.WithTimeout(context.Background(), defaultTestTimeout)
+	defer cancel()
+	ctx := stats.SetTags(tCtx, b)
 	if tg := stats.OutgoingTags(ctx); !reflect.DeepEqual(tg, b) {
 		t.Errorf("OutgoingTags(%v) = %v; want %v", ctx, tg, b)
 	}
@@ -1294,7 +1319,7 @@ func (s) TestTags(t *testing.T) {
 		t.Errorf("Tags(%v) = %v; want nil", ctx, tg)
 	}
 
-	ctx = stats.SetIncomingTags(context.Background(), b)
+	ctx = stats.SetIncomingTags(tCtx, b)
 	if tg := stats.Tags(ctx); !reflect.DeepEqual(tg, b) {
 		t.Errorf("Tags(%v) = %v; want %v", ctx, tg, b)
 	}
@@ -1305,7 +1330,9 @@ func (s) TestTags(t *testing.T) {
 
 func (s) TestTrace(t *testing.T) {
 	b := []byte{5, 2, 4, 3, 1}
-	ctx := stats.SetTrace(context.Background(), b)
+	tCtx, cancel := context.WithTimeout(context.Background(), defaultTestTimeout)
+	defer cancel()
+	ctx := stats.SetTrace(tCtx, b)
 	if tr := stats.OutgoingTrace(ctx); !reflect.DeepEqual(tr, b) {
 		t.Errorf("OutgoingTrace(%v) = %v; want %v", ctx, tr, b)
 	}
@@ -1313,7 +1340,7 @@ func (s) TestTrace(t *testing.T) {
 		t.Errorf("Trace(%v) = %v; want nil", ctx, tr)
 	}
 
-	ctx = stats.SetIncomingTrace(context.Background(), b)
+	ctx = stats.SetIncomingTrace(tCtx, b)
 	if tr := stats.Trace(ctx); !reflect.DeepEqual(tr, b) {
 		t.Errorf("Trace(%v) = %v; want %v", ctx, tr, b)
 	}
@@ -1321,3 +1348,153 @@ func (s) TestTrace(t *testing.T) {
 		t.Errorf("OutgoingTrace(%v) = %v; want nil", ctx, tr)
 	}
 }
+
+func (s) TestMultipleClientStatsHandler(t *testing.T) {
+	h := &statshandler{}
+	tc := &testConfig{compress: ""}
+	te := newTest(t, tc, []stats.Handler{h, h}, nil)
+	te.startServer(&testServer{})
+	defer te.tearDown()
+
+	cc := &rpcConfig{success: false, failfast: false, callType: unaryRPC}
+	_, _, err := te.doUnaryCall(cc)
+	if cc.success != (err == nil) {
+		t.Fatalf("cc.success: %v, got error: %v", cc.success, err)
+	}
+	te.cc.Close()
+	te.srv.GracefulStop() // Wait for the server to stop.
+
+	for start := time.Now(); time.Since(start) < defaultTestTimeout; {
+		h.mu.Lock()
+		if _, ok := h.gotRPC[len(h.gotRPC)-1].s.(*stats.End); ok && len(h.gotRPC) == 12 {
+			h.mu.Unlock()
+			break
+		}
+		h.mu.Unlock()
+		time.Sleep(10 * time.Millisecond)
+	}
+
+	for start := time.Now(); time.Since(start) < defaultTestTimeout; {
+		h.mu.Lock()
+		if _, ok := h.gotConn[len(h.gotConn)-1].s.(*stats.ConnEnd); ok && len(h.gotConn) == 4 {
+			h.mu.Unlock()
+			break
+		}
+		h.mu.Unlock()
+		time.Sleep(10 * time.Millisecond)
+	}
+
+	// Each RPC generates 6 stats events on the client-side, times 2 StatsHandler
+	if len(h.gotRPC) != 12 {
+		t.Fatalf("h.gotRPC: unexpected amount of RPCStats: %v != %v", len(h.gotRPC), 12)
+	}
+
+	// Each connection generates 4 conn events on the client-side, times 2 StatsHandler
+	if len(h.gotConn) != 4 {
+		t.Fatalf("h.gotConn: unexpected amount of ConnStats: %v != %v", len(h.gotConn), 4)
+	}
+}
+
+func (s) TestMultipleServerStatsHandler(t *testing.T) {
+	h := &statshandler{}
+	tc := &testConfig{compress: ""}
+	te := newTest(t, tc, nil, []stats.Handler{h, h})
+	te.startServer(&testServer{})
+	defer te.tearDown()
+
+	cc := &rpcConfig{success: false, failfast: false, callType: unaryRPC}
+	_, _, err := te.doUnaryCall(cc)
+	if cc.success != (err == nil) {
+		t.Fatalf("cc.success: %v, got error: %v", cc.success, err)
+	}
+	te.cc.Close()
+	te.srv.GracefulStop() // Wait for the server to stop.
+
+	for start := time.Now(); time.Since(start) < defaultTestTimeout; {
+		h.mu.Lock()
+		if _, ok := h.gotRPC[len(h.gotRPC)-1].s.(*stats.End); ok {
+			h.mu.Unlock()
+			break
+		}
+		h.mu.Unlock()
+		time.Sleep(10 * time.Millisecond)
+	}
+
+	for start := time.Now(); time.Since(start) < defaultTestTimeout; {
+		h.mu.Lock()
+		if _, ok := h.gotConn[len(h.gotConn)-1].s.(*stats.ConnEnd); ok {
+			h.mu.Unlock()
+			break
+		}
+		h.mu.Unlock()
+		time.Sleep(10 * time.Millisecond)
+	}
+
+	// Each RPC generates 6 stats events on the server-side, times 2 StatsHandler
+	if len(h.gotRPC) != 12 {
+		t.Fatalf("h.gotRPC: unexpected amount of RPCStats: %v != %v", len(h.gotRPC), 12)
+	}
+
+	// Each connection generates 4 conn events on the server-side, times 2 StatsHandler
+	if len(h.gotConn) != 4 {
+		t.Fatalf("h.gotConn: unexpected amount of ConnStats: %v != %v", len(h.gotConn), 4)
+	}
+}
+
+// TestStatsHandlerCallsServerIsRegisteredMethod tests whether a stats handler
+// gets access to a Server on the server side, and thus the method that the
+// server owns which specifies whether a method is made or not. The test sets up
+// a server with a unary call and full duplex call configured, and makes an RPC.
+// Within the stats handler, asking the server whether unary or duplex method
+// names are registered should return true, and any other query should return
+// false.
+func (s) TestStatsHandlerCallsServerIsRegisteredMethod(t *testing.T) {
+	wg := sync.WaitGroup{}
+	wg.Add(1)
+	stubStatsHandler := &testutils.StubStatsHandler{
+		TagRPCF: func(ctx context.Context, _ *stats.RPCTagInfo) context.Context {
+			// OpenTelemetry instrumentation needs the passed in Server to determine if
+			// methods are registered in different handle calls in to record metrics.
+			// This tag RPC call context gets passed into every handle call, so can
+			// assert once here, since it maps to all the handle RPC calls that come
+			// after. These internal calls will be how the OpenTelemetry instrumentation
+			// component accesses this server and the subsequent helper on the server.
+			server := internal.ServerFromContext.(func(context.Context) *grpc.Server)(ctx)
+			if server == nil {
+				t.Errorf("stats handler received ctx has no server present")
+			}
+			isRegisteredMethod := internal.IsRegisteredMethod.(func(*grpc.Server, string) bool)
+			// /s/m and s/m are valid.
+			if !isRegisteredMethod(server, "/grpc.testing.TestService/UnaryCall") {
+				t.Errorf("UnaryCall should be a registered method according to server")
+			}
+			if !isRegisteredMethod(server, "grpc.testing.TestService/FullDuplexCall") {
+				t.Errorf("FullDuplexCall should be a registered method according to server")
+			}
+			if isRegisteredMethod(server, "/grpc.testing.TestService/DoesNotExistCall") {
+				t.Errorf("DoesNotExistCall should not be a registered method according to server")
+			}
+			if isRegisteredMethod(server, "/unknownService/UnaryCall") {
+				t.Errorf("/unknownService/UnaryCall should not be a registered method according to server")
+			}
+			wg.Done()
+			return ctx
+		},
+	}
+	ss := &stubserver.StubServer{
+		UnaryCallF: func(ctx context.Context, in *testpb.SimpleRequest) (*testpb.SimpleResponse, error) {
+			return &testpb.SimpleResponse{}, nil
+		},
+	}
+	if err := ss.Start([]grpc.ServerOption{grpc.StatsHandler(stubStatsHandler)}); err != nil {
+		t.Fatalf("Error starting endpoint server: %v", err)
+	}
+	defer ss.Stop()
+
+	ctx, cancel := context.WithTimeout(context.Background(), defaultTestTimeout)
+	defer cancel()
+	if _, err := ss.Client.UnaryCall(ctx, &testpb.SimpleRequest{Payload: &testpb.Payload{}}); err != nil {
+		t.Fatalf("Unexpected error from UnaryCall: %v", err)
+	}
+	wg.Wait()
+}
