@@ -20,6 +20,7 @@ package resolver_test
 
 import (
 	"context"
+	"encoding/json"
 	"fmt"
 	"strings"
 	"testing"
@@ -34,7 +35,6 @@ import (
 	"google.golang.org/grpc/internal/grpcsync"
 	iresolver "google.golang.org/grpc/internal/resolver"
 	"google.golang.org/grpc/internal/testutils"
-	xdsbootstrap "google.golang.org/grpc/internal/testutils/xds/bootstrap"
 	"google.golang.org/grpc/internal/testutils/xds/e2e"
 	"google.golang.org/grpc/internal/xds/bootstrap"
 	"google.golang.org/grpc/metadata"
@@ -46,7 +46,6 @@ import (
 	"google.golang.org/grpc/xds/internal/httpfilter"
 	xdsresolver "google.golang.org/grpc/xds/internal/resolver"
 	rinternal "google.golang.org/grpc/xds/internal/resolver/internal"
-	xdstestutils "google.golang.org/grpc/xds/internal/testutils"
 	"google.golang.org/grpc/xds/internal/xdsclient"
 	"google.golang.org/grpc/xds/internal/xdsclient/xdsresource/version"
 	"google.golang.org/protobuf/proto"
@@ -87,14 +86,8 @@ func (s) TestResolverBuilder_ClientCreationFails_NoBootstrap(t *testing.T) {
 // not specified in the bootstrap file. Verifies that the resolver.Build method
 // fails with the expected error string.
 func (s) TestResolverBuilder_AuthorityNotDefinedInBootstrap(t *testing.T) {
-	bootstrapCleanup, err := xdsbootstrap.CreateFile(xdsbootstrap.Options{
-		NodeID:    "node-id",
-		ServerURI: "dummy-management-server",
-	})
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer bootstrapCleanup()
+	contents := e2e.DefaultBootstrapContents(t, "node-id", "dummy-management-server")
+	testutils.CreateBootstrapFileForTesting(t, contents)
 
 	builder := resolver.Get(xdsresolver.Scheme)
 	if builder == nil {
@@ -167,23 +160,30 @@ func (s) TestResolverResourceName(t *testing.T) {
 			mgmtServer, lisCh, _ := setupManagementServerForTest(ctx, t, nodeID)
 
 			// Create a bootstrap configuration with test options.
-			opts := xdsbootstrap.Options{
-				ServerURI: mgmtServer.Address,
+			opts := bootstrap.ConfigOptionsForTesting{
+				Servers: []json.RawMessage{[]byte(fmt.Sprintf(`{
+					"server_uri": %q,
+					"channel_creds": [{"type": "insecure"}]
+				}`, mgmtServer.Address))},
 				ClientDefaultListenerResourceNameTemplate: tt.listenerResourceNameTemplate,
+				Node: []byte(fmt.Sprintf(`{"id": "%s"}`, nodeID)),
 			}
 			if tt.extraAuthority != "" {
 				// In this test, we really don't care about having multiple
 				// management servers. All we need to verify is whether the
 				// resource name matches expectation.
-				opts.Authorities = map[string]string{
-					tt.extraAuthority: mgmtServer.Address,
+				opts.Authorities = map[string]json.RawMessage{
+					tt.extraAuthority: []byte(fmt.Sprintf(`{
+						"server_uri": %q,
+						"channel_creds": [{"type": "insecure"}]
+					}`, mgmtServer.Address)),
 				}
 			}
-			cleanup, err := xdsbootstrap.CreateFile(opts)
+			contents, err := bootstrap.NewContentsForTesting(opts)
 			if err != nil {
-				t.Fatal(err)
+				t.Fatalf("Failed to create bootstrap configuration: %v", err)
 			}
-			defer cleanup()
+			testutils.CreateBootstrapFileForTesting(t, contents)
 
 			buildResolverForTarget(t, resolver.Target{URL: *testutils.MustParseURL(tt.dialTarget)})
 			waitForResourceNames(ctx, t, lisCh, tt.wantResourceNames)
@@ -202,7 +202,7 @@ func (s) TestResolverWatchCallbackAfterClose(t *testing.T) {
 	// closed.
 	routeConfigResourceNamesCh := make(chan []string, 1)
 	waitForResolverCloseCh := make(chan struct{})
-	mgmtServer, err := e2e.StartManagementServer(e2e.ManagementServerOptions{
+	mgmtServer := e2e.StartManagementServer(t, e2e.ManagementServerOptions{
 		OnStreamRequest: func(_ int64, req *v3discoverypb.DiscoveryRequest) error {
 			if req.GetTypeUrl() == version.V3RouteConfigURL {
 				select {
@@ -218,21 +218,11 @@ func (s) TestResolverWatchCallbackAfterClose(t *testing.T) {
 			return nil
 		},
 	})
-	if err != nil {
-		t.Fatalf("Failed to start xDS management server: %v", err)
-	}
-	defer mgmtServer.Stop()
 
 	// Create a bootstrap configuration specifying the above management server.
 	nodeID := uuid.New().String()
-	cleanup, err := xdsbootstrap.CreateFile(xdsbootstrap.Options{
-		NodeID:    nodeID,
-		ServerURI: mgmtServer.Address,
-	})
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer cleanup()
+	contents := e2e.DefaultBootstrapContents(t, nodeID, mgmtServer.Address)
+	testutils.CreateBootstrapFileForTesting(t, contents)
 
 	// Configure resources on the management server.
 	listeners := []*v3listenerpb.Listener{e2e.DefaultClientListener(defaultTestServiceName, defaultTestRouteConfigName)}
@@ -257,17 +247,18 @@ func (s) TestResolverWatchCallbackAfterClose(t *testing.T) {
 
 // Tests that the xDS resolver's Close method closes the xDS client.
 func (s) TestResolverCloseClosesXDSClient(t *testing.T) {
-	bootstrapCfg := &bootstrap.Config{
-		XDSServer: xdstestutils.ServerConfigForAddress(t, "dummy-management-server-address"),
-	}
-
 	// Override xDS client creation to use bootstrap configuration pointing to a
 	// dummy management server. Also close a channel when the returned xDS
 	// client is closed.
 	origNewClient := rinternal.NewXDSClient
 	closeCh := make(chan struct{})
-	rinternal.NewXDSClient = func() (xdsclient.XDSClient, func(), error) {
-		c, cancel, err := xdsclient.NewWithConfigForTesting(bootstrapCfg, defaultTestTimeout, defaultTestTimeout)
+	rinternal.NewXDSClient = func(string) (xdsclient.XDSClient, func(), error) {
+		bc := e2e.DefaultBootstrapContents(t, uuid.New().String(), "dummy-management-server-address")
+		c, cancel, err := xdsclient.NewForTesting(xdsclient.OptionsForTesting{
+			Name:               t.Name(),
+			Contents:           bc,
+			WatchExpiryTimeout: defaultTestTimeout,
+		})
 		return c, grpcsync.OnceFunc(func() {
 			close(closeCh)
 			cancel()
